/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

/*
 * @(#) EjbcContext.java
 *
 */
package com.sun.ejb.codegen;

import java.io.File;
import java.util.*;

import com.sun.enterprise.deployment.backend.DeploymentMode;
import com.sun.enterprise.deployment.backend.DeploymentRequest;
import com.sun.enterprise.deployment.Application;


/**
 * Ejbc runtime environment created by deployment backend. This class 
 * contains all the information needed by ejbc for a particular deployment. 
 *
 * @author Nazrul Islam
 * @since  JDK 1.4
 */
public interface EjbcContext {

    /**
     * Returns the current directory where the archive has been exploded.
     *
     * @return    the directory where the archive has been exploded
     */
    public File getSrcDir();

    /**
     * Returns the old directory where the ear file was exploded (in case 
     * of redeployment) or null.
     *
     * @return    old directory where the ear file was exploded or null
     */
    public File getOldSrcDir();

    /**
     * Returns the (current stubs) directory where all the code is 
     * generated and being compiled for this archive.
     *
     * @return  the current stubs directory for this archive
     */
    public File getStubsDir();

    /**
     * Returns the (old stubs) directory where generated code from 
     * previous deployment can be found in case of redeployment. 
     *
     * @return  in case of redeployment, the old stubs directory or null
     */
    public File getOldStubsDir();

    /**
     * Returns the object representation of the deployment descriptor 
     * for the current archive.
     *
     * @return    the deployment descriptor for the current archive
     */
    public Application getDescriptor();

    /**
     * Returns true when the archive is being redeployed. 
     *
     * @return  true when the archive is being redeployed
     */
    public boolean isRedeploy();

    /** 
     * Return a class loader capable of loading classes from the old
     * repository for this app.  This will only include the application
     * classes, not the generated(e.g. stub) classes.
     */
    public ClassLoader getOldClassLoader();

    /**
     * Returns the class paths need by this archive to compile the 
     * generated src. 
     *
     * @return    the class paths used by javac & rmic for this archive
     */
    public String[] getClasspathUrls();

    /**
     * Returns the ejb class paths of this archive.
     *
     * @return    the ejb class paths for this archive
     */
    public String[] getEjbClasspathUrls();

    /**
     * Returns the RMIC options as defined in the instance's server
     * configuration. 
     *
     * @return   the RMIC options
     */
    public List getRmicOptions();

    /**
     * Returns the JAVAC options as defined in the instance's server
     * configuration. 
     * 
     * @return  the JAVAC options
     */
    public List getJavacOptions();

    /**
     * Returns the timing information for the sub-tasks of ejbc.
     * 
     * @return  timing information for the sub-tasks of ejbc
     */
    public IASEJBCTimes getTiming();

    /**
     * Returns the deployment mode, i.e., archive layout.
     *
     * @return    deployment mode
     */
    public DeploymentMode getDeploymentMode();
    /**
     * Returns the optional arguments - which currently consists of 
	 * CMP-specific deployment arguments
     *
     * @return    optional attributes
     */
    public Properties getOptionalArguments();
    /**
     * Returns the original DeploymentRequest object for the current deployment
     *
     * @return    Deployment Request object
     */
    public DeploymentRequest getDeploymentRequest();
    
    /**
     * Release any resources that are no longer needed, e.g. 
     * the class loader for the old repository during a redeployment.
     */
    public void cleanup();
}
