/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.deployment.backend;

import java.util.Map;
import java.util.Hashtable;
import java.util.logging.Level;

import com.sun.enterprise.util.i18n.StringManager;

/**
 * All client jar file are created in a separate thread when the client
 * jar file is not requested at deployment time. These threads are 
 * registered in this singleton registry so that when client jar files are 
 * requested from the deployment clients, we check if the client jar 
 * file is not in the process of being created.
 *
 * @author Jerome Dochez
 */
public class ClientJarMakerRegistry {
    
    // I am a singleton class
    private static ClientJarMakerRegistry theRegistry;
    
    // the registered client jar creator threads
    Map registeredThreads=null;
    
    /** Creates a new instance of ClientJarMakerRegistry */
    protected ClientJarMakerRegistry() {
        // I use hashtable since it needs to synchronized
        registeredThreads = new Hashtable();
    }
    
    /**
     * @return the singleton instance of ClientJarMakerRegistry
     */
    public static ClientJarMakerRegistry getInstance() {
        
        if (theRegistry==null) {
            synchronized(ClientJarMakerRegistry.class) {
                if (theRegistry==null) {
                    theRegistry = new ClientJarMakerRegistry();
                }
            }
        }
        return theRegistry;
    }
    
    
    /**
     * Register a new thread in the registry 
     * @param the module ID we are creating the client jar for
     * @param the thread object responsible for creating the 
     * client jar file
     */
    public void register(String moduleID, Thread clientJarMaker) {
        
        registeredThreads.put(moduleID, clientJarMaker);
    }
    
    /**
     * @return true if the passed module ID has a registered thread
     */
    public boolean isRegistered(String moduleID) {
        
        return registeredThreads.containsKey(moduleID);
    }
    
    /**
     * Unregister a thread in the registry. This is done when the 
     * thread has finished its execution
     * @param the module ID identifying the module this thread was
     * creating the client jar for.
     */
    public void unregister(String moduleID) {
        
        registeredThreads.remove(moduleID);
    }
    
    /**
     * wait for a particular thread maker to finish process before 
     * returning
     */
    public void waitForCompletion(String moduleID) {
        
        Thread maker = (Thread) registeredThreads.get(moduleID);
        if (maker==null) 
            return;
        
        try {
            maker.join();
        } catch(InterruptedException e) {
            StringManager localStrings = StringManager.getManager( ClientJarMakerRegistry.class );            
            DeploymentLogger.get().log(Level.SEVERE, 
                localStrings.getString("enterprise.deployment.error_creating_client_jar", 
                    e.getLocalizedMessage()) ,e);            
        }
        
        return;
    }
    
}
