/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.enterprise.distributedtx;

import java.rmi.RemoteException;
import java.io.Serializable;
import javax.transaction.*;

import com.sun.ejb.*;
import com.sun.enterprise.*;
import com.sun.enterprise.log.Log;
import com.sun.enterprise.util.i18n.StringManager;

import java.util.logging.*;
import com.sun.logging.*;

/** 
 * This class implements javax.transaction.UserTransaction .
 * Its methods are called from TX_BEAN_MANAGED EJB code.
 * Most of its methods just delegate to the TransactionManager
 * after doing some EJB Container-related steps.
 *
 * Note: EJB1.1 Section 6.4.1 requires that the Container must be able to 
 * preserve an object reference of the UserTransaction interface across 
 * passivation, so we make this Serializable.
 *
 * @author Tony Ng
 */

public class UserTransactionImpl implements UserTransaction, Serializable
{

    static Logger _logger=LogDomains.getLogger(LogDomains.JTA_LOGGER);
	// Sting Manager for Localization
	private static StringManager sm = StringManager.getManager(UserTransactionImpl.class);
    private static final boolean debug = false;
    private transient J2EETransactionManager transactionManager;
    private transient InvocationManager invocationManager;
    private transient boolean initialized;

    // for non-J2EE clients usage
    private transient UserTransaction userTx;

    // private int transactionTimeout;

    // true if ejb access checks should be performed.  Default is
    // true.  All instances of UserTransaction exposed to applications
    // will have checking turned on.   
    private boolean checkEjbAccess;

    /**
     * Default constructor.
     */
    public UserTransactionImpl()
    {
        this(true);
    }

    /**
     * Alternate version of constructor that allows control over whether
     * ejb access checks are performed.  
     */
    public UserTransactionImpl(boolean doEjbAccessChecks)
    {
        init();
        checkEjbAccess = doEjbAccessChecks;
    }


    /**
     * Could be called after passivation and reactivation
     */
    private void init()
    {
        initialized = true;
        Switch theSwitch = Switch.getSwitch();
        transactionManager = theSwitch.getTransactionManager();
        invocationManager = theSwitch.getInvocationManager();
        if (transactionManager == null) {
            // non J2EE client, set up UserTransaction from JTS
            userTx = new com.sun.jts.jta.UserTransactionImpl();
        }
    }

    private void checkUserTransactionMethodAccess(ComponentInvocation inv) 
        throws IllegalStateException, SystemException
    {
        if ( (inv.getInvocationType() == ComponentInvocation.EJB_INVOCATION)
             && checkEjbAccess ) {
            Container ejbContainer = (Container) inv.container;
            if( !ejbContainer.userTransactionMethodsAllowed(inv) ) {
                throw new IllegalStateException(sm.getString("enterprise_distributedtx.operation_not_allowed"));
            }
        } else if (inv.getInvocationType() == 
                   ComponentInvocation.APP_CLIENT_INVOCATION) {
            throw new SystemException(sm.getString("enterprise_distributedtx.usertransaction_not_supported"));
        }
    }

    public void begin() throws NotSupportedException, SystemException
    {
        if (!initialized) init();

        if (userTx != null) {
            userTx.begin();
            return;
        }

        ComponentInvocation inv = invocationManager.getCurrentInvocation();
	    if (inv != null) {
            checkUserTransactionMethodAccess(inv);
	    } else {
            throw new InvocationException();
	    }

	    transactionManager.begin();
            /**
	    if ( transactionTimeout > 0 ) 
	        transactionManager.begin(transactionTimeout);
	    else
	        transactionManager.begin();
            **/

	    try {
	        if ( inv.getInvocationType() == ComponentInvocation.EJB_INVOCATION )
		        ((Container)inv.container).doAfterBegin(inv);

            //transactionManager.preInvoke(null, cc); // enlist resources
            inv.setTransaction(transactionManager.getTransaction());
            transactionManager.enlistComponentResources();
        } catch ( RemoteException ex ) {
            _logger.log(Level.SEVERE,"enterprise_distributedtx.excep_in_utx_begin", ex);
            SystemException sysEx = new SystemException(ex.getMessage());
            sysEx.initCause(ex);
            throw sysEx;
        }
    }

    public void commit() throws RollbackException,
        HeuristicMixedException, HeuristicRollbackException, SecurityException,
        IllegalStateException, SystemException
    {
        if (!initialized) init();

        if (userTx != null) {
            userTx.commit();
            return;
        }

	    ComponentInvocation inv = invocationManager.getCurrentInvocation();
        if (inv != null) {
            checkUserTransactionMethodAccess(inv);
        } else {
            throw new InvocationException();
        }


        try {
            transactionManager.delistComponentResources(false);  // TMSUCCESS
            transactionManager.commit();
        } catch ( RemoteException ex ) {
            _logger.log(Level.SEVERE,"enterprise_distributedtx.excep_in_utx_commit", ex);
            throw new SystemException();
        } finally {
            inv.setTransaction(null);
        }
    }

    public void rollback() throws IllegalStateException, SecurityException,
        SystemException
    {
        if (!initialized) init();

        if (userTx != null) {
            userTx.rollback();
            return;
        }

        ComponentInvocation inv = invocationManager.getCurrentInvocation();
        if (inv != null) {
                checkUserTransactionMethodAccess(inv);
        } else {
            throw new InvocationException();
        }


        try {
            transactionManager.delistComponentResources(false); // TMSUCCESS
            transactionManager.rollback();
        } catch ( RemoteException ex ) {
            _logger.log(Level.SEVERE,"enterprise_distributedtx.excep_in_utx_rollback", ex);
            throw new SystemException();
        } finally {
            inv.setTransaction(null);
        }
    }

    public void setRollbackOnly() throws IllegalStateException, SystemException
    {
        if (!initialized) init();

        if (userTx != null) {
            userTx.setRollbackOnly();
            return;
        }

        ComponentInvocation inv = invocationManager.getCurrentInvocation();
        if (inv != null) {
                checkUserTransactionMethodAccess(inv);
        } else {
            throw new InvocationException();
        }

        transactionManager.setRollbackOnly();
    }

    public int getStatus() throws SystemException
    {
        if (!initialized) init();

        if (userTx != null) {
            return userTx.getStatus();
        }

        ComponentInvocation inv = invocationManager.getCurrentInvocation();
        if (inv != null) {
                checkUserTransactionMethodAccess(inv);
        } else {
            throw new InvocationException();
        }

        return transactionManager.getStatus();
    }

    public void setTransactionTimeout(int seconds) throws SystemException
    {
        if (!initialized) init();

        if (userTx != null) {
            userTx.setTransactionTimeout(seconds);
            return;
        }

	    ComponentInvocation inv = invocationManager.getCurrentInvocation();
        if (inv != null) {
                checkUserTransactionMethodAccess(inv);
        } else {
            throw new InvocationException();
        }

        
        if (seconds < 0) seconds = 0;
        // transactionTimeout = seconds;
        transactionManager.setTransactionTimeout(seconds);
    }
}
