/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.enterprise.server.ss.provider;

import java.io.EOFException;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.net.Socket;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sun.logging.LogDomains;


/**
 * OutputStream implementation for the socket wrappers of Quick startup
 * implementation. Implementation is thread safe while using nio to read
 * and write simultaneusly in the non-blocking mode.
 */

class ASOutputStream extends OutputStream{

    private static Logger logger = LogDomains.getLogger(LogDomains.CORE_LOGGER);

    private SocketChannel sc = null;
    private Socket sock = null;
    private Selector selector = null;
    private boolean closed = false;
    private ByteBuffer bb = null;
    private byte[] bs = null; 	// Invoker's previous array
    private byte[] b1 = null;

    ASOutputStream(SocketChannel sc, Socket sock) throws IOException {
        this.sc = sc;
        this.sock = sock;
        selector = Selector.open();
    }

    public void close() throws IOException {
        if (closed) {
            return;
        }
        closed = true;
        try {
            selector.close();
            selector = null;
            sc = null;
        } catch (Exception ie) {
            if ( logger.isLoggable(Level.FINE) ) {
                logger.log(Level.FINE, "" + ie.getMessage(), ie);
            }
        }
    }

    public void flush() throws IOException {
        checkClosed();
    }

    public synchronized void write(int b) throws IOException {
        if (b1 == null)
            b1 = new byte[1];
        b1[0] = (byte)b;
        this.write(b1);
    }

    public synchronized void write(byte[] bs, int off, int len) throws IOException {
        checkClosed();
        if ((off < 0) || (off > bs.length) || (len < 0) ||
           ((off + len) > bs.length) || ((off + len) < 0)) {
            throw new IndexOutOfBoundsException();
        } else if (len == 0) {
            return;
        }
        ByteBuffer bb = ((this.bs == bs)
                        ? this.bb
                        : ByteBuffer.wrap(bs));
        bb.limit(Math.min(off + len, bb.capacity()));
        bb.position(off);
        this.bb = bb;
        this.bs = bs;
        writeIt(bb);
    }
    
    private void writeIt(ByteBuffer bb) throws IOException {
         int attempts = 0;
         SelectionKey key = null;
         try {
        while (bb.hasRemaining()) {
                 int len = sc.write(bb);
                 attempts++;
                 
                 if (len < 0){
                     throw new EOFException();
        }
                 
                 if (len == 0) {
                     key = sc.register(selector, key.OP_WRITE);
                     if (selector.select(30000) == 0) {
                         if (attempts > 2)
                             throw new IOException("Client disconnected");
                     } else {
                         attempts--;
    }
                 } else {
                     attempts = 0;
                 }
             }
         } finally {
             try {
                 if (key != null) {
                     key.cancel();
                     key = null;
                     selector.selectNow();
                 }
             } catch (Exception e) {
             }
         }
     }

    protected void finalize() throws Throwable {
        try {
            close();
        } catch (Throwable t) {}
    }

    private void checkClosed() throws IOException {
        if (closed) {
            throw new IOException("Stream is closed");
        }
        
        if (sock.isOutputShutdown()) {
            throw new IOException("Output Shutdown");
        }
    }
}

