/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.gjc.spi;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionRequestInfo;
import com.sun.gjc.common.DataSourceSpec;
import com.sun.gjc.common.DataSourceObjectBuilder;
import com.sun.gjc.util.SecurityUtils;
import javax.resource.spi.security.PasswordCredential;
import javax.resource.spi.ResourceAllocationException;
import com.sun.gjc.spi.ManagedConnectionFactory;

import com.sun.logging.*;
import java.util.logging.Logger;
import java.util.logging.Level;
import com.sun.enterprise.util.i18n.StringManager;

/**
 * XA <code>ManagedConnectionFactory</code> implementation for Generic JDBC Connector.
 *
 * @version	1.0, 02/07/27
 * @author	Evani Sai Surya Kiran
 */

public class XAManagedConnectionFactory extends ManagedConnectionFactory {
    
    private transient javax.sql.XADataSource xaDataSourceObj;
    
    private static Logger _logger;
    static {
        _logger = LogDomains.getLogger( LogDomains.RSR_LOGGER );
    }
    
    /**
     * Creates a new physical connection to the underlying EIS resource
     * manager.
     * 
     * @param	subject	<code>Subject</code> instance passed by the application server
     * @param	cxRequestInfo	<code>ConnectionRequestInfo</code> which may be created
     *       	             	as a result of the invocation <code>getConnection(user, password)</code>
     *       	             	on the <code>DataSource</code> object
     * @return	<code>ManagedConnection</code> object created
     * @throws	ResourceException	if there is an error in instantiating the
     *        	                 	<code>DataSource</code> object used for the
     *       				creation of the <code>ManagedConnection</code> object
     * @throws	SecurityException	if there ino <code>PasswordCredential</code> object
     *        	                 	satisfying this request 
     * @throws	ResourceAllocationException	if there is an error in allocating the 
     *						physical connection
     */
    public javax.resource.spi.ManagedConnection createManagedConnection(javax.security.auth.Subject subject,
        ConnectionRequestInfo cxRequestInfo) throws ResourceException {
        if(logWriter != null) {
                logWriter.println("In createManagedConnection");
        }
        PasswordCredential pc = SecurityUtils.getPasswordCredential(this, subject, cxRequestInfo);
        
        if(xaDataSourceObj == null) {
            if(dsObjBuilder == null) {
                dsObjBuilder = new DataSourceObjectBuilder(spec);
            }
        
            try {
                xaDataSourceObj = (javax.sql.XADataSource) dsObjBuilder.constructDataSourceObject();
            } catch(ClassCastException cce) {
	        _logger.log(Level.SEVERE, "jdbc.exc_cce_XA", cce);
                throw new javax.resource.ResourceException(cce.getMessage());
            }
        }
        
        javax.sql.XAConnection xaConn = null;
        
        try {
	    /* For the case where the user/passwd of the connection pool is
	     * equal to the PasswordCredential for the connection request
	     * get a connection from this pool directly.
	     * for all other conditions go create a new connection
	     */
	    if ( isEqual( pc, getUser(), getPassword() ) ) {
	        xaConn = xaDataSourceObj.getXAConnection();
	    } else {
	        xaConn = xaDataSourceObj.getXAConnection(pc.getUserName(), 
		    new String(pc.getPassword()));
	    }


        } catch(java.sql.SQLException sqle) {
	    //_logger.log(Level.WARNING, "jdbc.exc_create_xa_conn",sqle);
	    _logger.log(Level.FINE, "jdbc.exc_create_xa_conn",sqle);
            StringManager sm = StringManager.getManager( 
                DataSourceObjectBuilder.class );
	    String msg = sm.getString( "jdbc.cannot_allocate_connection", sqle.getMessage() );
	    ResourceAllocationException rae = new ResourceAllocationException(
	        msg, sqle );
	    throw rae;	
        }
        
        com.sun.gjc.spi.ManagedConnection mc = constructManagedConnection(
            xaConn, null, pc, this );
        
	mc.initializeConnectionType( ManagedConnection.ISXACONNECTION);
        //GJCINT
	validateAndSetIsolation( mc );
        return mc;
    }
    
    /**
     * Check if this <code>ManagedConnectionFactory</code> is equal to 
     * another <code>ManagedConnectionFactory</code>.
     * 
     * @param	other	<code>ManagedConnectionFactory</code> object for checking equality with
     * @return	true	if the property sets of both the 
     *			<code>ManagedConnectionFactory</code> objects are the same
     *		false	otherwise
     */
    public boolean equals(Object other) {
        if(logWriter != null) {
                logWriter.println("In equals");
        }
        /**
         * The check below means that two ManagedConnectionFactory objects are equal
         * if and only if their properties are the same.
         */
        if(other instanceof com.sun.gjc.spi.XAManagedConnectionFactory) {
            com.sun.gjc.spi.XAManagedConnectionFactory otherMCF = 
                (com.sun.gjc.spi.XAManagedConnectionFactory) other;
            return this.spec.equals(otherMCF.spec);
        }
        return false;
    }
    
    
    /**
     * Sets the server name.
     *
     * @param	serverName	<code>String</code>
     * @see	<code>getServerName</code>
     */
    public void setserverName(String serverName) {
        spec.setDetail(DataSourceSpec.SERVERNAME, serverName);
    }
    
    /**
     * Gets the server name.
     *
     * @return	serverName
     * @see	<code>setServerName</code>
     */
    public String getserverName() {
        return spec.getDetail(DataSourceSpec.SERVERNAME);
    }
    
    /**
     * Sets the server name.
     *
     * @param	serverName	<code>String</code>
     * @see	<code>getServerName</code>
     */
    public void setServerName(String serverName) {
        spec.setDetail(DataSourceSpec.SERVERNAME, serverName);
    }
    
    /**
     * Gets the server name.
     *
     * @return	serverName
     * @see	<code>setServerName</code>
     */
    public String getServerName() {
        return spec.getDetail(DataSourceSpec.SERVERNAME);
    }
    
    /**
     * Sets the port number.
     *
     * @param	portNumber	<code>String</code>
     * @see	<code>getPortNumber</code>
     */
    public void setportNumber(String portNumber) {
        spec.setDetail(DataSourceSpec.PORTNUMBER, portNumber);
    }
    
    /**
     * Gets the port number.
     *
     * @return	portNumber
     * @see	<code>setPortNumber</code>
     */
    public String getportNumber() {
        return spec.getDetail(DataSourceSpec.PORTNUMBER);
    }
    
    /**
     * Sets the port number.
     *
     * @param	portNumber	<code>String</code>
     * @see	<code>getPortNumber</code>
     */
    public void setPortNumber(String portNumber) {
        spec.setDetail(DataSourceSpec.PORTNUMBER, portNumber);
    }
    
    /**
     * Gets the port number.
     *
     * @return	portNumber
     * @see	<code>setPortNumber</code>
     */
    public String getPortNumber() {
        return spec.getDetail(DataSourceSpec.PORTNUMBER);
    }
    
    /**
     * Sets the database name.
     *
     * @param	databaseName	<code>String</code>
     * @see	<code>getDatabaseName</code>
     */
    public void setdatabaseName(String databaseName) {
        spec.setDetail(DataSourceSpec.DATABASENAME, databaseName);
    }
    
    /**
     * Gets the database name.
     *
     * @return	databaseName
     * @see	<code>setDatabaseName</code>
     */
    public String getdatabaseName() {
        return spec.getDetail(DataSourceSpec.DATABASENAME);
    }
    
    /**
     * Sets the database name.
     *
     * @param	databaseName	<code>String</code>
     * @see	<code>getDatabaseName</code>
     */
    public void setDatabaseName(String databaseName) {
        spec.setDetail(DataSourceSpec.DATABASENAME, databaseName);
    }
    
    /**
     * Gets the database name.
     *
     * @return	databaseName
     * @see	<code>setDatabaseName</code>
     */
    public String getDatabaseName() {
        return spec.getDetail(DataSourceSpec.DATABASENAME);
    }
    
    /**
     * Sets the data source name.
     *
     * @param	dsn <code>String</code>
     * @see	<code>getDataSourceName</code>
     */
    public void setdataSourceName(String dsn) {
        spec.setDetail(DataSourceSpec.DATASOURCENAME, dsn);
    }
    
    /**
     * Gets the data source name.
     *
     * @return	dsn
     * @see	<code>setDataSourceName</code>
     */
    public String getdataSourceName() {
        return spec.getDetail(DataSourceSpec.DATASOURCENAME);
    }
    
    /**
     * Sets the data source name.
     *
     * @param	dsn <code>String</code>
     * @see	<code>getDataSourceName</code>
     */
    public void setDataSourceName(String dsn) {
        spec.setDetail(DataSourceSpec.DATASOURCENAME, dsn);
    }
    
    /**
     * Gets the data source name.
     *
     * @return	dsn
     * @see	<code>setDataSourceName</code>
     */
    public String getDataSourceName() {
        return spec.getDetail(DataSourceSpec.DATASOURCENAME);
    }
    
    /**
     * Sets the description.
     *
     * @param	desc	<code>String</code>
     * @see	<code>getDescription</code>
     */
    public void setdescription(String desc) {
        spec.setDetail(DataSourceSpec.DESCRIPTION, desc);
    }
    
    /**
     * Gets the description.
     *
     * @return	desc
     * @see	<code>setDescription</code>
     */
    public String getdescription() {
        return spec.getDetail(DataSourceSpec.DESCRIPTION);
    }
    
    /**
     * Sets the description.
     *
     * @param	desc	<code>String</code>
     * @see	<code>getDescription</code>
     */
    public void setDescription(String desc) {
        spec.setDetail(DataSourceSpec.DESCRIPTION, desc);
    }
    
    /**
     * Gets the description.
     *
     * @return	desc
     * @see	<code>setDescription</code>
     */
    public String getDescription() {
        return spec.getDetail(DataSourceSpec.DESCRIPTION);
    }
    
    /**
     * Sets the network protocol.
     *
     * @param	nwProtocol	<code>String</code>
     * @see	<code>getNetworkProtocol</code>
     */
    public void setnetworkProtocol(String nwProtocol) {
        spec.setDetail(DataSourceSpec.NETWORKPROTOCOL, nwProtocol);
    }
    
    /**
     * Gets the network protocol.
     *
     * @return	nwProtocol
     * @see	<code>setNetworkProtocol</code>
     */
    public String getnetworkProtocol() {
        return spec.getDetail(DataSourceSpec.NETWORKPROTOCOL);
    }
    
    /**
     * Sets the network protocol.
     *
     * @param	nwProtocol	<code>String</code>
     * @see	<code>getNetworkProtocol</code>
     */
    public void setNetworkProtocol(String nwProtocol) {
        spec.setDetail(DataSourceSpec.NETWORKPROTOCOL, nwProtocol);
    }
    
    /**
     * Gets the network protocol.
     *
     * @return	nwProtocol
     * @see	<code>setNetworkProtocol</code>
     */
    public String getNetworkProtocol() {
        return spec.getDetail(DataSourceSpec.NETWORKPROTOCOL);
    }
    
    /**
     * Sets the role name.
     *
     * @param	roleName	<code>String</code>
     * @see	<code>getRoleName</code>
     */
    public void setroleName(String roleName) {
        spec.setDetail(DataSourceSpec.ROLENAME, roleName);
    }
    
    /**
     * Gets the role name.
     *
     * @return	roleName
     * @see	<code>setRoleName</code>
     */
    public String getroleName() {
        return spec.getDetail(DataSourceSpec.ROLENAME);
    }
    
    /**
     * Sets the role name.
     *
     * @param	roleName	<code>String</code>
     * @see	<code>getRoleName</code>
     */
    public void setRoleName(String roleName) {
        spec.setDetail(DataSourceSpec.ROLENAME, roleName);
    }
    
    /**
     * Gets the role name.
     *
     * @return	roleName
     * @see	<code>setRoleName</code>
     */
    public String getRoleName() {
        return spec.getDetail(DataSourceSpec.ROLENAME);
    }
    
    /**
     * Sets the max statements.
     *
     * @param	maxStmts	<code>String</code>
     * @see	<code>getMaxStatements</code>
     */
    public void setmaxStatements(String maxStmts) {
        spec.setDetail(DataSourceSpec.MAXSTATEMENTS, maxStmts);
    }
    
     /**
     * Gets the max statements.
     *
     * @return	maxStmts
     * @see	<code>setMaxStatements</code>
     */
    public String getmaxStatements() {
        return spec.getDetail(DataSourceSpec.MAXSTATEMENTS);
    }
    
    /**
     * Sets the max statements.
     *
     * @param	maxStmts	<code>String</code>
     * @see	<code>getMaxStatements</code>
     */
    public void setMaxStatements(String maxStmts) {
        spec.setDetail(DataSourceSpec.MAXSTATEMENTS, maxStmts);
    }
    
     /**
     * Gets the max statements.
     *
     * @return	maxStmts
     * @see	<code>setMaxStatements</code>
     */
    public String getMaxStatements() {
        return spec.getDetail(DataSourceSpec.MAXSTATEMENTS);
    }
    
    /**
     * Sets the initial pool size.
     *
     * @param	initPoolSz	<code>String</code>
     * @see	<code>getInitialPoolSize</code>
     */
    public void setinitialPoolSize(String initPoolSz) {
        spec.setDetail(DataSourceSpec.INITIALPOOLSIZE, initPoolSz);
    }
    
    /**
     * Gets the initial pool size.
     *
     * @return	initPoolSz
     * @see	<code>setInitialPoolSize</code>
     */
    public String getinitialPoolSize() {
        return spec.getDetail(DataSourceSpec.INITIALPOOLSIZE);
    }
    
    /**
     * Sets the initial pool size.
     *
     * @param	initPoolSz	<code>String</code>
     * @see	<code>getInitialPoolSize</code>
     */
    public void setInitialPoolSize(String initPoolSz) {
        spec.setDetail(DataSourceSpec.INITIALPOOLSIZE, initPoolSz);
    }
    
    /**
     * Gets the initial pool size.
     *
     * @return	initPoolSz
     * @see	<code>setInitialPoolSize</code>
     */
    public String getInitialPoolSize() {
        return spec.getDetail(DataSourceSpec.INITIALPOOLSIZE);
    }
    
    /**
     * Sets the minimum pool size.
     *
     * @param	minPoolSz	<code>String</code>
     * @see	<code>getMinPoolSize</code>
     */
    public void setminPoolSize(String minPoolSz) {
        spec.setDetail(DataSourceSpec.MINPOOLSIZE, minPoolSz);
    }
    
    /**
     * Gets the minimum pool size.
     *
     * @return	minPoolSz
     * @see	<code>setMinPoolSize</code>
     */
    public String getminPoolSize() {
        return spec.getDetail(DataSourceSpec.MINPOOLSIZE);
    }
    
    /**
     * Sets the minimum pool size.
     *
     * @param	minPoolSz	<code>String</code>
     * @see	<code>getMinPoolSize</code>
     */
    public void setMinPoolSize(String minPoolSz) {
        spec.setDetail(DataSourceSpec.MINPOOLSIZE, minPoolSz);
    }
    
    /**
     * Gets the minimum pool size.
     *
     * @return	minPoolSz
     * @see	<code>setMinPoolSize</code>
     */
    public String getMinPoolSize() {
        return spec.getDetail(DataSourceSpec.MINPOOLSIZE);
    }
    
    /**
     * Sets the maximum pool size.
     *
     * @param	maxPoolSz	<code>String</code>
     * @see	<code>getMaxPoolSize</code>
     */
    public void setmaxPoolSize(String maxPoolSz) {
        spec.setDetail(DataSourceSpec.MAXPOOLSIZE, maxPoolSz);
    }
    
    /**
     * Gets the maximum pool size.
     *
     * @return	maxPoolSz
     * @see	<code>setMaxPoolSize</code>
     */
    public String getmaxPoolSize() {
        return spec.getDetail(DataSourceSpec.MAXPOOLSIZE);
    }
    
    /**
     * Sets the maximum pool size.
     *
     * @param	maxPoolSz	<code>String</code>
     * @see	<code>getMaxPoolSize</code>
     */
    public void setMaxPoolSize(String maxPoolSz) {
        spec.setDetail(DataSourceSpec.MAXPOOLSIZE, maxPoolSz);
    }
    
    /**
     * Gets the maximum pool size.
     *
     * @return	maxPoolSz
     * @see	<code>setMaxPoolSize</code>
     */
    public String getMaxPoolSize() {
        return spec.getDetail(DataSourceSpec.MAXPOOLSIZE);
    }
    
    /**
     * Sets the maximum idle time.
     *
     * @param	maxIdleTime	String
     * @see	<code>getMaxIdleTime</code>
     */
    public void setmaxIdleTime(String maxIdleTime) {
        spec.setDetail(DataSourceSpec.MAXIDLETIME, maxIdleTime);
    }
    
    /**
     * Gets the maximum idle time.
     *
     * @return	maxIdleTime
     * @see	<code>setMaxIdleTime</code>
     */
    public String getmaxIdleTime() {
        return spec.getDetail(DataSourceSpec.MAXIDLETIME);
    }
    
    /**
     * Sets the maximum idle time.
     *
     * @param	maxIdleTime	String
     * @see	<code>getMaxIdleTime</code>
     */
    public void setMaxIdleTime(String maxIdleTime) {
        spec.setDetail(DataSourceSpec.MAXIDLETIME, maxIdleTime);
    }
    
    /**
     * Gets the maximum idle time.
     *
     * @return	maxIdleTime
     * @see	<code>setMaxIdleTime</code>
     */
    public String getMaxIdleTime() {
        return spec.getDetail(DataSourceSpec.MAXIDLETIME);
    }
    
    /**
     * Sets the property cycle.
     *
     * @param	propCycle	<code>String</code>
     * @see	<code>getPropertyCycle</code>
     */
    public void setpropertyCycle(String propCycle) {
        spec.setDetail(DataSourceSpec.PROPERTYCYCLE, propCycle);
    }
    
    /**
     * Gets the property cycle.
     *
     * @return	propCycle
     * @see	<code>setPropertyCycle</code>
     */
    public String getpropertyCycle() {
        return spec.getDetail(DataSourceSpec.PROPERTYCYCLE);
    }
    
    /**
     * Sets the property cycle.
     *
     * @param	propCycle	<code>String</code>
     * @see	<code>getPropertyCycle</code>
     */
    public void setPropertyCycle(String propCycle) {
        spec.setDetail(DataSourceSpec.PROPERTYCYCLE, propCycle);
    }
    
    /**
     * Gets the property cycle.
     *
     * @return	propCycle
     * @see	<code>setPropertyCycle</code>
     */
    public String getPropertyCycle() {
        return spec.getDetail(DataSourceSpec.PROPERTYCYCLE);
    }
}    
